/*'
' * Project name:
'     UV click - UV Click demo
' * Copyright:
'     (c) mikroElektronika, 2014.
' * Revision History:
'     20140827:
'       - Initial release (BD & TP);
' * Description:
'     UV click is a simple solution for adding an ultraviolet light sensor to your design.
'     It carries the ML8511 IC UV sensor with Voltage output and a MCP3201 ADC.
'     UV click can output either analog or digital signals (determined by the position of the A/D SEL jumper).
'     Therefore, it communicates with the target board either through mikroBUS SPI (CS, SCK, MISO), or AN lines; additionally,
'     the sensor can be enabled or disabled from the MCU through the RST (EN) pin.
'
'     UV click uses a 3.3V power supply!
'
'     With UV click, you can measure Ultraviolet light intensity both indoors and outdoors.
'     The photodiode on the ML8511 IC is sensitive to UV-A (365315 nm) and UV-B (315-280) rays.
'     You can use UV click to design devices that protect the user against excesive sun exposure,
'     or for industrial purposes where UV light is used for sterilization.
'     When using UV click, be careful not to rub or press the surface of the ML8511 IC.
'
'     To switch between analog and digital output options, resolder the onboard zero-ohm SMD jumper.
'     By default, it's soldered in the ADC position.
'
'     Design devices that warn you of excesive ultraviolet radiation levels with UV click.
'     This click boards carries the ML85111 IC that is sensitive to UV-A (365-315 nm) and UV-B (315-280 nm) rays.
'     The board can output either analog or digital signals proportional to the amount of UV exposure.
'     To bypass the onboard MCP3201 ADC and get an analog output, solder the A/D SEL jumper in the AN position.
'     Depending on the position of the jumper, UV click communicates with the target board either through mikroBUS SPI
'     (CS, SCK, MISO) or AN lines; additionally, the sensor can be enabled or disabled from the MCU through the RST (EN) pin.
'     When handling UV click, be careful not to press or rub the sensitive surface of the ML85111 sensor IC.
'     UV click uses a 3.3V power supply only.
'
'     In ML8511 datasheet please see graph on page 4.
'     Output values is voltage in mV representing UV intensity mW/cm2 @ 365nm. For more details please see sensor datasheet.
'
' * Test configuration:
'     MCU:             STM32F107VC
'                      http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATASHEET/CD00220364.pdf
'     Dev. Board:      EasyMx PRO v7 for STM32
'                      http://www.mikroe.com/eng/products/view/852/easymx-pro-v7-for-stm32/
'     Oscillator:      HS-PLL 72.0000 MHz, internal 8.0000 MHz RC
'     ext. modules:    UV click
'                      http://www.mikroe.com/click/uv/
'     SW:              mikroC PRO for ARM
'                      http://www.mikroe.com/mikroc/arm/
' * NOTES:
'     - Place UV Click in the mikroBUS socket 1.
'     - Enable SW12-1 and SW12-2 switches for USB UARTA
'     - Use USART Terminal from Tools menu at 56000bps speed.
*/

// UV Click connections
sbit UV_Enable at GPIOC_ODR.B2;
sbit CS_MCP3201 at GPIOD_ODR.B13;

// Init MCU function
void InitMCU(){

    GPIO_Digital_Output(&GPIOC_BASE, _GPIO_PINMASK_2);      // Set rst pin (GPIOC pin 3)as digital output
    GPIO_Digital_Output(&GPIOD_BASE, _GPIO_PINMASK_13);     // Set chip select (GPIOD pin 13) as digital output
    // Initialize SPI3 module
    SPI3_Init_Advanced(_SPI_FPCLK_DIV8, _SPI_MASTER | _SPI_8_BIT |
                       _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION |
                       _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1,
                       &_GPIO_MODULE_SPI3_PC10_11_12);
    UART1_Init(56000);                   // Initialize UART
    Delay_ms(100);                       // Wait for UART module to stabilize
    UV_Enable=1;                         // ENABLE UV click module
    CS_MCP3201 = 1;                      // Deselect MCP3204
    //Delay_100ms();                     // Wait for UART module to stabilize
}

// Get ADC values
unsigned int getADC(){
    volatile unsigned int adc_l = 0 , adc_h = 0, i = 0, tmp1=0, tmp2=0, avrg = 0;
    short buffer;

   for(i=0; i<16; i++ ){                    // Get 12 ADC values
            CS_MCP3201 = 0;                 // Select MCP3201
            adc_h = SPI3_Read(buffer);      // Get first 8 bits of ADC value
            adc_l = SPI3_Read(buffer);      // Get second 8 bits of ADC value
            CS_MCP3201= 1;                  // Deselect MCP3201
            // Format the acquired value
            tmp1 = ((adc_h & 0x1F) << 8);   // Store the first 8 bits of the ADC value in temporary variable
            tmp2 = ((tmp1 | adc_l) >> 1);   // Store the rest of the ADC value bits in temporary variable
            avrg += tmp2;                   // Sum the acquired values
   }
  avrg >>= 4;                               // Calculate the average value
  return avrg;                              // Returns 12-bit ADC result
}

// Process and display current value to UART
void processValue(unsigned int valueTemp) {
  float UV_val;
  float temp;
  char txt[15];

      if ((valueTemp > 1100) && (valueTemp < 4095)) {
                  UV_val = valueTemp * 3300.0 / 4096.0;         // Vref=3300 ; 12bit=4096 ;
                  UART1_Write_Text("UV sensor reading = ");
                  sprintf(txt, "%4.1f mV", UV_val);             // Format UV_val and store it to txt
                  UART1_Write_Text(txt);                        // Print UV value
                  UART1_Write(13);                              //
                  UART1_Write(10);                              // Add New line   CR+LF
      }
      else{
                  UART1_Write_Text("Error");                    // In case when ADC value is not in 800-4095 range
                  UART1_Write(13);                              //
                  UART1_Write(10);                              // Add New line   CR+LF
      }
}

void main(){
  unsigned int measurement;
  
  InitMCU();                                // Initialize MCU
  measurement = 0;                          // Initialize the measurement variable
  UART1_Write_Text("UV click Initialized"); // Check the UART communication with simple
  UART1_Write(13);
  UART1_Write(10);                          // Add New line   CR+LF

  while (1) {
       measurement = getADC();              // Get ADC result
       ProcessValue(measurement);           // Display UV sensor values via UART
       Delay_ms(500);                       // Wait
  }
}